#include "main.h"

//--Gobale Variablen
volatile double Temperatur=-55;			//--Temperatur
volatile double Drehzahl=-550;			//--Drehzahl

volatile unsigned char Display_Update=0;
volatile unsigned long compare = 0;	//--Anzahl berlufe
volatile unsigned long ic_zp_A = 0;	//--ICR-Wert bei 1.High-Flanke speichern
volatile unsigned long ic_zp_B = 0;	//--ICR-Wert bei 2.High-Flanke speichern
volatile unsigned char Sig_Counter = 0;	//--Signalzhler
volatile unsigned char Drehzahlupdate = 0;	//--Job Flag


static const PROGMEM unsigned char Omega[] =
{
	0x03, 0x04, 0x08, 0x08, 0x08, 0x04, 0x1E, 0x00,
	0x18, 0x04, 0x02, 0x02, 0x02, 0x04, 0x0F, 0x00
};

static const PROGMEM unsigned char HochMinus1[] =
{
	0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00,
	0x02, 0x06, 0x0A, 0x02, 0x02, 0x00, 0x00, 0x00
};

static const PROGMEM unsigned char Grad[] =
{
	0x06, 0x09, 0x09, 0x06, 0x00, 0x00, 0x00, 0x00,
};

//-------------------------------------------------------------------------
//--Funktion fr Sonderzeichen aufs Display
//-------------------------------------------------------------------------
void Sonderzeichen()
{  
	unsigned char i;

    lcd_command(_BV(LCD_CGRAM));  /* set CG RAM start address 0 */

	for(i=0; i<16; i++)
	{
	   lcd_data(pgm_read_byte_near(&Omega[i]));
	}

	for(i=0; i<16; i++)
	{
	   lcd_data(pgm_read_byte_near(&HochMinus1[i]));
	}

	for(i=0; i<8; i++)
	{
	   lcd_data(pgm_read_byte_near(&Grad[i]));
	}
}
//-------------------------------------------------------------------------
//--Funktion fr das Auslesen des AD Wandlers
//-------------------------------------------------------------------------
double get_AD(uint8_t mux)
{
  unsigned char i;
  double result;
 
  ADCSRA = (1<<ADEN) | (1<<ADPS1) | (1<<ADPS0);    // Frequenzvorteiler 
                               // setzen auf 8 (1) und ADC aktivieren (1)
 
  ADMUX = mux;                      // Kanal waehlen
  ADMUX |= (0<<REFS1) | (1<<REFS0); // Referenzspannung
 
  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
  ADCSRA |= (1<<ADSC);              // eine ADC-Wandlung 
  while ( ADCSRA & (1<<ADSC) )
  {
     ;     // auf Abschluss der Konvertierung warten 
  }
  result = ADCW;  // ADCW muss einmal gelesen werden,
                  // sonst wird Ergebnis der nchsten Wandlung
                  // nicht bernommen.
 
  /* Eigentliche Messung - Mittelwert aus aufeinanderfolgenden Wandlungen */
  result = 0; 
  for( i=0; i<10; i++ )
  {
    ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
    while ( ADCSRA & (1<<ADSC) ) 
	{
      ;   // auf Abschluss der Konvertierung warten
    }
    result += ADCW;		    // Wandlungsergebnisse aufaddieren
  }
  ADCSRA &= ~(1<<ADEN);             // ADC deaktivieren (2)
 
  result /= 10;                     // Summe durch vier teilen = arithm. Mittelwert
 
  return result;
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
//--Funktion fr das Auslesen der Temperatur
//-------------------------------------------------------------------------
double get_Temp()
{
	//--Hilfvariable
	double Rueckgabe=0;

	Rueckgabe=get_AD(0);

	//--Eichen auf 0 Grad C
	Rueckgabe = ((Rueckgabe-373.3895238)/4.414285714);

	return Rueckgabe;
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
//--Funktion fr das Updaten des Displays
//-------------------------------------------------------------------------
void Update_Display()
{
	unsigned char i=0;				//--Hilfsvariable 
	char string[10];				//--Hilfsvariable 

	//--Temperatur Updaten
	dtostrf(Temperatur,4,1,string);

	lcd_gotoxy(7,1);

	i=0;
	//--Ausgabe des Doublewertes aufs Display
	while( string[i] != '\0' )
	{
		lcd_putc( string[i++] );
	}
	//--Ausgabe Grad Celsius
	lcd_putc(4);
	lcd_puts("C");

	//--Drehzahlupdate
	dtostrf(Drehzahl,5,0,string);

	lcd_gotoxy(7,0);

	i=0;
	//--Ausgabe des Doublewertes aufs Display
	while( string[i] != '\0' )
	{
		lcd_putc( string[i++] );
	}

	lcd_puts("min");
	//-- Hoch minus 1 ausgeben
	lcd_putc(2);
	lcd_putc(3);

}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
//--Capture Event
//-------------------------------------------------------------------------
SIGNAL(SIG_INPUT_CAPTURE1)
{
	if( Drehzahlupdate == 1 )   // wenn die vorhergehende Messung
    {                         // noch nicht ausgewertet wurde
		return;	
	}
	
	if (Sig_Counter == 0)    // 1.High Flanke
	{
		ic_zp_A = ICR1;
		compare = 0;
		Sig_Counter = 1;
	}
	else         // 2.High Flanke
	{
		ic_zp_B = ICR1;
        // Messung ist fertig
		Drehzahlupdate = 1;
		Sig_Counter = 0;
    }
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
//--Timer 1 berlaufevent
//-------------------------------------------------------------------------
SIGNAL(SIG_OVERFLOW1)
{
  	compare++;

	//--zu oft Timeroverflow
	//--Drehzahl auf 0 setzten
	if(compare > 25)
	{
		Drehzahl = 0;
	}
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
//--Timer 0 berlaufevent
//-------------------------------------------------------------------------
SIGNAL(SIG_OVERFLOW0)
{
	Display_Update++;

	if(Display_Update>50)
	{
		Update_Display();
		Display_Update=0;
	}
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
//--MAIN FUNKTION
//-------------------------------------------------------------------------
int main(void)
{ 
	double Hilfsdrehzahl;
//	char string[10];					//--Hilfsvariable fr die bertragung
//	unsigned char i=0;				//--Hilfsvariable fr die bertragung

	//--Initialisierung der Pereferie
	/* initialize display, cursor off */
    lcd_init(LCD_DISP_ON);

	 /* clear display and home cursor */
    lcd_clrscr();

	//--Sonderzeichen ins Display Laden
	Sonderzeichen();

	//--Grundanzeige auf dem Display
	lcd_gotoxy(0,0);
	lcd_puts("Motor:");

	lcd_gotoxy(0,1);
	lcd_puts("Temp.:");

	//--Einstellungen fr Drehzahlmessung
	DDRD = (DDRD & 0xFD);	//--ICP Freischalten
	TCCR1B =  (1<<ICES1)  | (1<<CS12) | (1<<CS10); // Input Capture Edge, PreScale 1024 (ICP)

	//--Timer0 fr Display Aktualisierung
	TCCR0 = (1<<CS02) | (1<<CS00); 	//--1024 Vorteiler
	
	//--Timer Starten
	TIMSK = (1<<TOIE0) | (1<<TICIE1) | (1<<OCIE1A);;

	//--Erste Displayaktualisierung
	//	Update_Display();

	//--Globale Interrups Aktivieren
	sei();

	while(1)
	{
		Temperatur = get_Temp();

		//--Drehzahlupdate
		if(Drehzahlupdate == 1 )
		{	//--Neue Drehzahl berrechnen	
			Hilfsdrehzahl =	(compare*65536) + ic_zp_B - ic_zp_A;		
			Hilfsdrehzahl = (Hilfsdrehzahl* 64.0) / 1000000.0;
			Hilfsdrehzahl = 60/Hilfsdrehzahl;

			Drehzahl = Hilfsdrehzahl;

			//--Bereit fr neue Drehzahl
			Drehzahlupdate = 0;	
		}
	}
	return 0;
}//--Ende Mainfunktion
